@@ -318,6 +318,7 @@ def _parse_porcelain_blame(blame):
                 start_of_line_info = False
             elif entries[0] == 'filename':
                 filename = entries[1]
+    
     blame_info = pd.DataFrame(l)
     return blame_info
 
@@ -680,7 +681,7 @@ def _extract_edits(git_repo, commit, modification, extraction_settings):
             e['commit_hash'] = commit.hash
             e['modification_type'] = modification.change_type.name
             e['edit_type'] = edit.type
-
+            
             e.update(_get_edit_details(edit, commit, deleted_lines, added_lines, blame_info_parent,
                                       blame_info_commit, extraction_settings))
 
@@ -972,6 +973,15 @@ def _get_edited_file_paths_since_split(git_repo, commit):
     return edited_file_paths
 
 
+def _check_mailmap(name, email, git_repo):
+    test_str = '{} <{}>'.format(name, email)
+
+    out_str = git_repo.git.check_mailmap(test_str)
+    
+    name, email = re.findall("^(.*) <(.*)>$", out_str)[0]
+    return name, email
+
+
 def _process_commit(args):
     """ Extracts information on commit and all edits made with the commit.
 
@@ -993,13 +1003,21 @@ def _process_commit(args):
     alarm.start()
 
     try:
+        author_name, author_email = _check_mailmap(commit.author.name,
+                                                   commit.author.email,
+                                                   git_repo)
+
+        committer_name, committer_email = _check_mailmap(commit.committer.name,
+                                                         commit.committer.email,
+                                                         git_repo)
+
         # parse commit
         c = {}
         c['hash'] = commit.hash
-        c['author_email'] = commit.author.email
-        c['author_name'] = commit.author.name
-        c['committer_email'] = commit.committer.email
-        c['committer_name'] = commit.committer.name
+        c['author_email'] = author_email
+        c['author_name'] = author_name
+        c['committer_email'] = committer_email
+        c['committer_name'] = committer_name
         c['author_date'] = commit.author_date.strftime('%Y-%m-%d %H:%M:%S')
         c['committer_date'] = commit.committer_date.strftime('%Y-%m-%d %H:%M:%S')
         c['author_timezone'] = commit.author_timezone
@@ -1142,6 +1160,11 @@ def _process_repo_serial(git_repo_dir, sqlite_db_file, commits, extraction_setti
                 result['commit'].to_sql('commits', con, if_exists='append', index=False)
 
 
+# suggestion by marco-c (github.com/ishepard/pydriller/issues/110)
+def _init(git_repo_dir, git_init_lock_):
+    global git_init_lock
+    git_init_lock = git_init_lock_
+                
 def _process_repo_parallel(git_repo_dir, sqlite_db_file, commits, extraction_settings):
     """ Processes all commits in a given git repository in a parallel manner.
 
@@ -1158,11 +1181,6 @@ def _process_repo_parallel(git_repo_dir, sqlite_db_file, commits, extraction_set
     args = [{'git_repo_dir': git_repo_dir, 'commit_hash': commit.hash, 'extraction_settings': extraction_settings}
             for commit in commits]
 
-    # suggestion by marco-c (github.com/ishepard/pydriller/issues/110)
-    def _init(git_repo_dir, git_init_lock_):
-        global git_init_lock
-        git_init_lock = git_init_lock_
-
     with multiprocessing.Pool(extraction_settings['no_of_processes'],
                               initializer=_init, initargs=(git_repo_dir,git_init_lock)) as p:
         with tqdm(total=len(args), desc='Parallel ({0} processes)' \
